// Multiset.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

// O continer associativo 'multiset' oferece armazengaem e recuperao rpidas de chaves. 
// Um 'multiset' permite chaves duplicadas (elementos duplicados. A ordenao dos elementos 
//  determinada por um "objeto funo de comparao". 

// Por exemplo, em um 'multiset'de inteiros, os elementos podem ser ordenados em ordem 
// crescente ordenando-se as chaves com o objeto funo de comparao 'less<int>'. 

// O tipo de dado das chaves em todos os contineres associativos deve suportar
// adequadamente a comparao, com base no objeto de comparao especificado - as chaves 
// ordenadas com 'less<int>'devem suportar a comparao com 'operator<'. Se as chaves usadas
// em contineres associativos so de tipos definidos pelo programador, estes tipos devem prover
// os operadores de comprao apropriado. Um 'multiset'suporta iteradores bidirecionais 
//(mas no iteradires de acesso randmico).

// (Deitel & Deitel, pg. 943)

int _tmain(int argc, _TCHAR* argv[])
{
	const int TAMANHO = 10;
	int a[TAMANHO] = { 7, 22, 9, 1, 18, 30, 100, 22, 85, 90 };

	// Typedef para criar um novo nome de tipo (alias) para um multiset de inteiros ordenados 
	// em ordem ascendente usando o objeto funo less<int>. 
	typedef std::multiset< int, std::less< int > > ims;
	//typedef std::multiset< int > ims;

	// Este novo tipo  ento usado para instanciar um objeto integer Multiset,
	// denominado MultisetInteiro.
	ims MultisetInteiro;   // ims significa "integer multiset"
	
	// Define um iterador
	std::ostream_iterator< int > Saida (cout, " ");

	// Conta quantos quinzes existem no Multiset.
	cout << "Atualmente existem " << MultisetInteiro.count( 15 )
		 << " valores iguais a 15 no multiset." << endl;
	
	// Insere quinzes no Multiset.
	MultisetInteiro.insert( 15 );
	MultisetInteiro.insert( 15 );

	// Conta quantos quinzes existem no Multiset.
    cout << "Aps inserts, existem " << MultisetInteiro.count( 15 )
		 << " valores iguais a 15 no multiset." << endl;

	// Definio de um Iterador 
	ims::const_iterator Resultado; 

	// Procura um elemento retornando-o para o iterador.
	Resultado = MultisetInteiro.find (15);  // find retorna iterador

	// Se o interador no est no fim do Multiset...
	if (Resultado != MultisetInteiro.end()) // se o iterador no est no fim
	{
		cout << "Encontrou o valor 15." << endl; // ento encontrou o valor 15.
	}

	// ...
	MultisetInteiro.insert( 20 );
	Resultado = MultisetInteiro.find( 20 );
	if (Resultado == MultisetInteiro.end())
	{
		cout	<< "No encontrou o valor 20." << endl; 
	}
	else
	{
		cout	<< "E nao  que encontrei a tal do 20!" << endl;
	}
	cout << endl;


    // Insere os elementos do vetor a no MultisetInteiro...
	// insere e os ordena cf. definido no ims
	MultisetInteiro.insert( a, a + TAMANHO );
	cout	<< "Aps insert MultisetInteiro contm: " << endl << endl;
	
	// O algoritmo 'copia' os elementos do MultisetInteiro para a Saida padro.
	std::copy(MultisetInteiro.begin(), MultisetInteiro.end(), Saida);
	cout	<< endl << endl;

	// Encontra a primeira ocorrencia do elemento 22.
	cout	<< "Limite inferior de 9: " << *(MultisetInteiro.lower_bound( 9 ) ) << endl;
	// Encontra o primeiro elemento aps a ltima ocorrncia de 22...
	// (Ambas as funes retornam 'iterators' ou 'const_iterators' apontando para posio apropriada ou
	// o 'iterator' retornado por 'end', se o valor procurado no est no 'multiset'
	cout	<< "Limite superior de 9: " << *(MultisetInteiro.upper_bound( 9 ) ) << endl;

	cout << endl;

	// Instancia uma instncia da classe 'pair' chamada p. Objetos da classe 'pair' so usados para 
	// associar pares de valores. Neste exemplo, os contedos de um pair so dois 'const_iterators'
	// para nosso 'multiset' baseado em inteiros. A finalidade de p  armazenar o valor retornado
	// pela funo 'equal_range' de 'multiset' que retorna um 'pair' contendo os resultados 
	// das operaes 'lower_bound' e 'upper_bound'. O tipo 'pair' contm dois membros de dados
	// 'public' chamado 'first' e 'second'.
	std::pair< ims::const_iterator, ims::const_iterator > p;

	// Usa a funo equal_range para determinar o 'lower_bound' e o 'upper_bound'de 22 no 
	// 'multiset'. 'P.first' e 'P.second'acessam respectivamente o 'lower_bound'e o 'upper_bound'.
	// Derreferenciamos os iteradores para enviar para a sada valores retornados por 'equal_range'

	p = MultisetInteiro.equal_range( 22 );

	cout	<< "Usando equal_range de 22." << endl
			<< "       Limite inferior:  " << *( p.first ) << endl
			<< "       Limite superior:  " << *( p.second )<< endl;

	return 0;
}

